use dioxus::prelude::*;

use crate::components::layout::icons;
use crate::util::model::member::Member;
use std::collections::HashMap;

pub fn Auth() -> Element {
    let mut register = use_signal(|| false);

    rsx! {
        div {
            class: "flex flex-col items-center",
            h1 { class: "font-bold text-primary text-center text-2xl mt-20", "Waddinxveense Reddingsbrigade" },
            div {
                class: "card bg-base-200 mt-20 w-full max-w-lg shadow-xl",
                div {
                    class: "card-body",
                    div {
                        role: "tablist",
                        class: "tabs tabs-boxed bg-base-300 mx-auto w-full mb-10",
                        button {
                            onclick: move |_| register.set(false),
                            role: "tab",
                            class: "tab",
                            class: if !register() { "tab-active" },
                            "Inloggen"
                        },
                        button {
                            onclick: move |_| register.set(true),
                            role: "tab",
                            class: "tab",
                            class: if register() { "tab-active" },
                            "Registreren"
                        },
                    }
                    if !register() {
                        Login { }
                    } else {
                        Register { }
                    }
                }
            }
        }
    }
}

fn Login() -> Element {
    rsx! {
        form {
            label {
                class: "form-control w-full",
                div {
                    class: "label",
                    span { class: "label-text", "Email" },
                }
                input {
                    r#type: "email",
                    class: "input input-bordered w-full",
                },
            }
            label {
                class: "form-control w-full mt-5",
                div {
                    class: "label",
                    span { class: "label-text", "Password" },
                }
                input {
                    r#type: "password",
                    class: "input input-bordered w-full",
                },
            }
            div {
                class: "card-actions mt-8",
                button {
                    class: "btn btn-primary",
                    "Inloggen"
                }
            }
        }
    }
}

fn Register() -> Element {
    let mut members: Signal<HashMap<String, Member>> = use_signal(|| HashMap::new());
    let mut input_registration_token = use_signal(|| "".to_string());

    rsx! {
        label {
            class: "form-control w-full join-item",
            div {
                class: "label",
                span { class: "label-text", "Registratie code" },
            }
            div {
                class: "w-full join",
                input {
                    r#type: "text",
                    class: "input input-bordered w-full join-item",
                    value: "{input_registration_token}",
                    oninput: move |event| input_registration_token.set(event.value())
                },
                button {
                    class: "btn join-item btn-neutral",
                    onclick: move |_| async move {
                        if let Ok(res) = fetch_member(input_registration_token()).await {
                            members.write().insert(res.id.clone(), res);
                            input_registration_token.set("".to_string());
                        }
                    },
                    icons::UserPlus {},
                    "Toevoegen",
                }
            }
            div {
                class: "flex gap-1 mt-2 flex-wrap pb-3",
                for (id, member) in members() {
                    div {
                        class: "badge badge-md badge-neutral h-8 gap-1 hover:cursor-pointer hover:line-through whitespace-nowrap",
                        onclick: move |_| {
                            members.write().remove(&id);
                        },
                        "{member.name.full}"
                        icons::XMark {},
                    }
                }
            }
        }
        div {
            class: "card-actions mt-8",
            button {
                class: "btn btn-primary",
                "Volgende"
            }
        }
    }
}

#[server]
async fn fetch_member(registration_token: String) -> Result<Member, ServerFnError> {
    let member = Member::fetch_from_registration_token(registration_token).await?;

    match member {
        Some(m) => Ok(m),
        None => Err(ServerFnError::new("Lid niet gevonden")),
    }
}
